/*
  alt_sem.h - override for Altera's os/alt_sem.h

  Part of FreeRTOS NIOS2 port - Copyright (C) 2006 Kolja Waschk

  This port is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This port is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  A special exception to the GPL can be applied should you wish to distribute
  a combined work that includes this port, without being obliged to provide
  the source code for any proprietary components.  See the licensing section 
  of http://www.FreeRTOS.org for full details of how and when the exception
  can be applied.

  You should have received a copy of the GNU General Public License
  along with FreeRTOS.org; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __ALT_SEM_H__
#define __ALT_SEM_H__

#include "FreeRTOS.h"
#include "portmacro.h"
#include "semphr.h"

#define ALT_SEM(sem)        xSemaphoreHandle sem;
#define ALT_EXTERN_SEM(sem) extern xSemaphoreHandle sem;
#define ALT_STATIC_SEM(sem) static xSemaphoreHandle sem;

static ALT_INLINE int ALT_ALWAYS_INLINE ALT_SEM_POST(
  xSemaphoreHandle sem
)
{
  return xSemaphoreGive(sem);
}

static ALT_INLINE int ALT_ALWAYS_INLINE ALT_SEM_PEND(
  xSemaphoreHandle sem, 
  portTickType timeout)
{
  if(timeout==0) 
  {
    while(xSemaphoreTake(sem, portMAX_DELAY) == pdFAIL);
    return 0;
  }
  return xSemaphoreTake(sem, timeout);
}

static ALT_INLINE int ALT_ALWAYS_INLINE ALT_SEM_CREATE(
  xSemaphoreHandle *sem, 
  int value)
{
#if 0
  xSemaphoreHandle lsem;
  /* TODO: handle value != 1 */
  
  vSemaphoreCreateBinary( lsem );
  *sem = lsem;
#else
  vSemaphoreCreateBinary( *sem );
#endif
  return 0;
}

             
#endif /* __ALT_SEM_H__ */
