/*
  portmacro.h - macro implementation of routines defined in portable.h 

  Part of FreeRTOS NIOS2 port - Copyright (C) 2006 Kolja Waschk

  This port is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This port is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with FreeRTOS.org; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
  A special exception to the GPL can be applied should you wish to distribute
  a combined work that includes this port, without being obliged to provide
  the source code for any proprietary components.  See the licensing section 
  of http://www.FreeRTOS.org for full details of how and when the exception
  can be applied.

  ***************************************************************************
  This file has been derived from the MicroBlaze portmacro.h from the
  FreeRTOS.org distribution 4.0.4; Copyright (C) 2003-2006 R. Barry.
  See http://www.FreeRTOS.org for documentation, latest information, license 
  and contact details.  Please ensure to read the configuration and relevant 
  port sections of the online documentation.
  ***************************************************************************
*/

#ifndef PORTMACRO_H
#define PORTMACRO_H

#include <sys/alt_irq.h>

/*-----------------------------------------------------------
 * Port specific definitions.  
 *
 * The settings in this file configure FreeRTOS correctly for the
 * given hardware and compiler.
 *
 * These settings should not be altered.
 *-----------------------------------------------------------
 */

/* Type definitions. */
#define portCHAR       char
#define portFLOAT      float
#define portDOUBLE     double
#define portLONG       long
#define portSHORT      short
#define portSTACK_TYPE unsigned portLONG
#define portBASE_TYPE  portLONG

#if( configUSE_16_BIT_TICKS == 1 )
	typedef unsigned portSHORT portTickType;
	#define portMAX_DELAY ( portTickType ) 0xffffU
#else
	typedef unsigned portLONG portTickType;
	#define portMAX_DELAY ( portTickType ) 0xffffffffUL
#endif
/*-----------------------------------------------------------*/	

/* Interrupt control macros. */
#define portDISABLE_INTERRUPTS()	(void)alt_irq_disable_all();
#define portENABLE_INTERRUPTS()		alt_irq_enable_all(1);

/*-----------------------------------------------------------*/

/* Critical section macros. */
extern void vPortEnterCritical( void );
extern void vPortExitCritical( void );

#define portENTER_CRITICAL() \
  { \
    extern unsigned portBASE_TYPE uxCriticalNesting; \
    portDISABLE_INTERRUPTS(); \
    uxCriticalNesting++; \
  }
									
#define portEXIT_CRITICAL() \
  { \
    extern unsigned portBASE_TYPE uxCriticalNesting; \
    /* Interrupts are disabled, so we can */\
    /* access the variable directly. */ \
    uxCriticalNesting--; \
    if( uxCriticalNesting == 0 ) \
    {  \
      /* The nesting has unwound and we \
         can enable interrupts again. */ \
      portENABLE_INTERRUPTS();\
    }\
  }

/*-----------------------------------------------------------*/

/* Task utilities. */
extern void vPortYield( void );
extern void vTaskSwitchContext();

#define portYIELD()          vPortYield()
#define portYIELD_FROM_ISR() vTaskSwitchContext()

/*-----------------------------------------------------------*/

/* Hardware specifics. */
#define portBYTE_ALIGNMENT  4
#define portSTACK_GROWTH    (-1)
#define portTICK_RATE_MS    ((portTickType)1000 / configTICK_RATE_HZ)
#define portNOP()           asm volatile ( "nop" )

/*-----------------------------------------------------------*/

/* Task function macros as described on the FreeRTOS.org WEB site. */
#define portTASK_FUNCTION_PROTO( vFunction, pvParameters ) void vFunction( void *pvParameters )
#define portTASK_FUNCTION( vFunction, pvParameters ) void vFunction( void *pvParameters )

#endif /* PORTMACRO_H */

